/**
 *  Copyright (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  Licensed under the GPL, Version 3.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *       http://www.gnu.org/licenses/gpl.html
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * --------------------------------------------------------------------------
 *  版权 (c) 1997-2013, tinygroup.org (luo_guo@live.cn).
 *
 *  本开源软件遵循 GPL 3.0 协议;
 *  如果您不遵循此协议，则不被允许使用此文件。
 *  你可以从下面的地址获取完整的协议文本
 *
 *       http://www.gnu.org/licenses/gpl.html
 */
package org.tinygroup.factory.fileresolver;

import java.util.ArrayList;
import java.util.List;

import org.tinygroup.factory.BeanFactory;
import org.tinygroup.factory.config.Beans;
import org.tinygroup.factory.impl.SimpleFactory;
import org.tinygroup.fileresolver.impl.AbstractFileProcessor;
import org.tinygroup.logger.LogLevel;
import org.tinygroup.logger.Logger;
import org.tinygroup.logger.LoggerFactory;
import org.tinygroup.vfs.FileObject;
import org.tinygroup.xstream.XStreamFactory;

import com.thoughtworks.xstream.XStream;

public class BeansFileRemoveProcessor extends AbstractFileProcessor {
	private static Logger logger = LoggerFactory
			.getLogger(BeansFileRemoveProcessor.class);
	private List<FileObject> fileObjects = new ArrayList<FileObject>();

	public boolean isMatch(FileObject fileObject) {
		return fileObject.getFileName().endsWith(".beans.xml");
	}

	private void process(FileObject fileObject) {
		XStream xStream = XStreamFactory.getXStream();
		try {
			logger.logMessage(LogLevel.INFO, "正在加载Bean描述文件：[{}]",
					fileObject.getAbsolutePath());
			Beans beans = (Beans) xStream.fromXML(fileObject.getInputStream());
			if (BeanFactory.getFactory() instanceof SimpleFactory) {
				SimpleFactory simpleFactory = (SimpleFactory) BeanFactory
						.getFactory();
				simpleFactory.removeBeans(beans);
			}
			logger.logMessage(LogLevel.INFO, "Bean描述文件：[{}]加载成功。",
					fileObject.getAbsolutePath());
		} catch (Exception e) {
			logger.errorMessage("加载bean配置文件[{0}]时发生错误。", e,
					fileObject.getAbsolutePath());
		}
	}

	public void process() {
		if (BeanFactory.getFactory() instanceof SimpleFactory) {
			for (FileObject fileObject : fileObjects) {
				process(fileObject);
			}
		}
	}

}
